﻿/*
 * Asteroids4k
 *
 * An entry for the 2009 KMK Game Programming Contest
 * Written by Soothsayer in May 2009
 * 
 * Website: http://www.cppanddelphi.atw.hu
 * Contact: cppanddelphi@gmail.com
 */
package 
{
  import GameObject;
  import flash.geom.Point;
  import flash.geom.Matrix;
  import flash.display.Sprite;
  import flash.text.TextField;

  public class Game extends Sprite 
  {
    public var gameStarted: Boolean = false;
    public var keyUP: Boolean;
    public var keyLEFT: Boolean;
    public var keyRIGHT: Boolean;
    public var keySPACE: Boolean;
    public var keyENTER: Boolean;

    public var ship: GameObject;
    public var shield: GameObject;
    public var time: Number = 0;
    public var bulletList: Array;
    public var asteroidList: Array;
    public var damage: GameObject;
    public var life: GameObject;
    public var power: Number;
    public var gameLevel: Number;
    public var score: Number;
    public var bestScore: Number;

    public var txtScore:*;
    public var txtTitle:*;
    public var txtStart:*;

    public function Game() 
    {
      var i: Number;

      asteroidList = new Array(90);
      for (i = 0; i < asteroidList.length; i++)
      {
        asteroidList[i] = new GameObject(3);
        asteroidList[i].visible = false;
        addChild(asteroidList[i]);
      }

      bulletList = new Array(10);
      for (i = 0; i < bulletList.length; i++)
      {
        bulletList[i] = new GameObject(2);
        bulletList[i].visible = false;
        addChild(bulletList[i]);
      }

      shield = new GameObject(1);
      addChild(shield);

      ship = new GameObject(0);
      addChild(ship);

      damage = new GameObject(4);
      addChild(damage);
      damage.x = 530;
      damage.y = 10;

      life = new GameObject(5);
      addChild(life);
      life.x = 530;
      life.y = 10;

      bestScore = 0;

      txtScore = new TextField();
      txtScore.width = 250;
      txtScore.height = 100;
      txtScore.textColor = 0xFFFFFF;
      addChild(txtScore);
      txtScore.x = 5;
      txtScore.y = 5;

      txtTitle = new TextField();
      txtTitle.width = 250;
      txtTitle.height = 100;
      txtTitle.textColor = 0xFFFFFF;
      addChild(txtTitle);
      txtTitle.x = 535;
      txtTitle.y = 5;
      txtTitle.text = "ASTEROIDS 4K";

      txtStart = new TextField();
      txtStart.width = 250;
      txtStart.height = 100;
      txtStart.textColor = 0xFFFFFF;
      addChild(txtStart);
      txtStart.x = 267;
      txtStart.y = 195;
      txtStart.text = "ENTER TO START";

      addEventListener("enterFrame", run);
      stage.addEventListener("keyDown", keyHandle);
      stage.addEventListener("keyUp", keyHandle);
    }

    public function CreateAsteroid(pointX: Number, pointY: Number, level: Number): void
    {
      var i: Number;

      if (level >= 0)
      {
        for (i = 0; i < asteroidList.length; i++)
        {
          if (! asteroidList[i].visible)
          break;
        }
        if (level == 2)
        {
          asteroidList[i].pos.x = Math.random() * 640;
          asteroidList[i].pos.y = Math.random() * 400;
        }
        else
        {
          asteroidList[i].pos.x = pointX + (Math.random() * 15 + 15) * Sign();
          asteroidList[i].pos.y = pointY + (Math.random() * 15 + 15) * Sign();
        }
        asteroidList[i].angle = Math.random() * 360;
        asteroidList[i].vel.x = (Math.random() * 25 + 25) * Sign();
        asteroidList[i].vel.y = (Math.random() * 25 + 25) * Sign();
        asteroidList[i].angVel = (Math.random() * 40 + 40) * Sign();
        asteroidList[i].level = level;
        asteroidList[i].visible = true;
        asteroidList[i].time = 0;
        asteroidList[i].matrix.createBox(1 << asteroidList[i].level, 1 << asteroidList[i].level, asteroidList[i].angle * 0.0174532, asteroidList[i].pos.x, asteroidList[i].pos.y);
        asteroidList[i].transform.matrix = asteroidList[i].matrix;
      }
    }

    public function CreateGame(level: Number): void 
    {
      var i: Number;

      ship.pos.x = 320;
      ship.pos.y = 200;
      ship.angle = -90;
      ship.vel.x = 0;
      ship.vel.y = 0;
      ship.angVel = 0;
      ship.time = 0;

      for (i = 0; i < asteroidList.length; i++) 
        asteroidList[i].visible = false;

      time = 0;
      for (i = 0; i < bulletList.length; i++) 
        bulletList[i].visible = false;

      shield.visible = true;
      ship.visible = true;
      damage.visible = true;
      life.visible = true;

      for (i = 0; i < level; i++)
        CreateAsteroid(0, 0, 2);
    }

    public function Sign(): Number 
    {
      if (Math.random() < 0.5) return -1; else return 1;
    }

    public function Clipping(obj: GameObject): void 
    {
      if (obj.pos.x < -25) obj.pos.x = 665;
      if (obj.pos.x > 665) obj.pos.x = -25;
      if (obj.pos.y < -25) obj.pos.y = 425;
      if (obj.pos.y > 425) obj.pos.y = -25;
    }

    public function SetDirection(obj: GameObject): void 
    {
      obj.dir.x = Math.cos(obj.angle * 0.0174532); 
      obj.dir.y = Math.sin(obj.angle * 0.0174532);
    }

    public function Update(obj: GameObject, delta: Number): void 
    {
      obj.time += delta;
      delta /= 1000;
      obj.vel.x += delta * obj.acc.x;
      obj.vel.y += delta * obj.acc.y;
      obj.pos.x += delta * obj.vel.x;
      obj.pos.y += delta * obj.vel.y;
      obj.angle += delta * obj.angVel;
    }

    public function Collide(objAsteroid: GameObject, objShip: GameObject): Boolean 
    {
      var i: Number;
      var axx, axy, ayx, ayy: Number;
      var transx, transy: Number;

      axx = objShip.dir.x;
      axy = objShip.dir.y;
      ayx = -axy;
      ayy = axx;
      transx = objShip.pos.x + axx * 10 + ayx * 0;
      transy = objShip.pos.y + axy * 10 + ayy * 0;
      if (objAsteroid.hitTestPoint(transx, transy, true)) return true;
      transx = objShip.pos.x + axx * -10 + ayx * 5;
      transy = objShip.pos.y + axy * -10 + ayy * 5;
      if (objAsteroid.hitTestPoint(transx, transy, true)) return true;
      transx = objShip.pos.x + axx * -10 + ayx * -5;
      transy = objShip.pos.y + axy * -10 + ayy * -5;
      if (objAsteroid.hitTestPoint(transx, transy, true)) return true;
      return false;
    }

    public final function keyHandle(e:*):*
    {
      if (e.keyCode == 38) keyUP = (e.type == "keyDown");
      if (e.keyCode == 37) keyLEFT = (e.type == "keyDown");
      if (e.keyCode == 39) keyRIGHT = (e.type == "keyDown");
      if (e.keyCode == 32) keySPACE = (e.type == "keyDown");
      if (e.keyCode == 13) keyENTER = (e.type == "keyDown");
    }	

    public function keyPressed(): void 
    {
      var i: Number;

      if (keyLEFT) ship.angVel = -150;;
      if (keyRIGHT) ship.angVel = 150;
      if (! keyLEFT && ! keyRIGHT) ship.angVel = 0;
      if (keyUP) 
      {
        ship.acc.x = Math.cos(ship.angle * 0.0174532) * 100; 
        ship.acc.y = Math.sin(ship.angle * 0.0174532) * 100;
      }
      if (! keyUP) 
      {
        ship.acc.x = 0; 
        ship.acc.y = 0;
      }

      for (i = 0; i < bulletList.length; i++)
        if (bulletList[i].time > 1000)
          bulletList[i].visible = false;
      if (keySPACE)
      {
        for (i = 0; i < bulletList.length; i++)
        {
          if (bulletList[i].visible) continue;

          if (time > 100 && ! bulletList[i].visible)
          {
            time = 0;
            bulletList[i].pos.x = ship.pos.x; 
            bulletList[i].pos.y = ship.pos.y;
            bulletList[i].dir.x = ship.dir.x; 
            bulletList[i].dir.y = ship.dir.y;
            bulletList[i].angle = ship.angle;
            bulletList[i].vel.x = ship.vel.x + ship.dir.x * 200; 
            bulletList[i].vel.y = ship.vel.y + ship.dir.y * 200;
            bulletList[i].time = 0;
            bulletList[i].visible = true;
            bulletList[i].matrix.createBox(1, 1, bulletList[i].angle * 0.0174532, bulletList[i].pos.x, bulletList[i].pos.y);
            bulletList[i].transform.matrix = bulletList[i].matrix;
            break;
          }
        }
      }
    }

    public final function run(e:*):*
    {
      var i: Number;
      var j: Number;
      var k: Number;
      var l: Number;

      if (! gameStarted && keyENTER) 
      {
        score = 0;
        power = 1;
        gameLevel = 3;
        CreateGame(gameLevel);
        gameStarted = true;
        txtTitle.visible = false;
        txtStart.visible = false;
      }
      if (gameStarted)
      {
        keyPressed();

        for (i = 0; i < asteroidList.length; i++)
        {
          if (! asteroidList[i].visible) continue;

          Clipping(asteroidList[i]);
          SetDirection(asteroidList[i]);
          Update(asteroidList[i], 20);
          asteroidList[i].matrix.createBox(1 << asteroidList[i].level, 1 << asteroidList[i].level, asteroidList[i].angle * 0.0174532, asteroidList[i].pos.x, asteroidList[i].pos.y);
          asteroidList[i].transform.matrix = asteroidList[i].matrix;
        }

        for (i = 0; i < bulletList.length; i++)
        {
          if (bulletList[i].visible)
          {
            Clipping(bulletList[i]);
            SetDirection(bulletList[i]);
            Update(bulletList[i], 20);
          }
          bulletList[i].matrix.createBox(1, 1, bulletList[i].angle * 0.0174532, bulletList[i].pos.x, bulletList[i].pos.y);
          bulletList[i].transform.matrix = bulletList[i].matrix;
        }
        time += 20;

        if (ship.time < 8000)
        {
          shield.pos.x = ship.pos.x;
          shield.pos.y = ship.pos.y;
          Clipping(shield);
        }
        else
        {
          shield.visible = false;

          for (i = 0; i < asteroidList.length; i++)
          {
            if (! asteroidList[i].visible) continue;

            if (Collide(asteroidList[i], ship))
            {
              power -= 0.02;
              if (power <= 0) gameStarted = false;
            }
          }
        }
        shield.matrix.createBox(1, 1, 0, shield.pos.x, shield.pos.y);
        shield.transform.matrix = shield.matrix;

        Clipping(ship);
        SetDirection(ship);
        Update(ship, 20);
        ship.matrix.createBox(1, 1, ship.angle * 0.0174532, ship.pos.x, ship.pos.y);
        ship.transform.matrix = ship.matrix;

        life.matrix.createBox(power, 1, 0, life.x, life.y);
        life.transform.matrix = life.matrix;

        l = 0;
        for (i = 0; i < asteroidList.length; i++)
        {
          if (! asteroidList[i].visible) continue;

          l++; 
          for (j = 0; j < bulletList.length; j++)
          {
            if (! bulletList[j].visible) continue;

            if (asteroidList[i].hitTestPoint(bulletList[j].pos.x, bulletList[j].pos.y, true))
            {   
              asteroidList[i].visible = false;
              bulletList[j].visible = false;
              score += 100 * (3 - asteroidList[i].level);
              k = asteroidList[i].level - 1;
              CreateAsteroid(asteroidList[i].pos.x, asteroidList[i].pos.y, k);
              CreateAsteroid(asteroidList[i].pos.x, asteroidList[i].pos.y, k);
              CreateAsteroid(asteroidList[i].pos.x, asteroidList[i].pos.y, k);
            }
          }
        }

        if (l == 0)
        {
          if (gameLevel < 10)
          {
            gameLevel++;
            CreateGame(gameLevel);
          }
          else gameStarted = false;
        }

        txtScore.text = "SCORE: " + int(score);
      }
      else
      {
        txtTitle.visible = true;
        txtStart.visible = true;

        for (i = 0; i < asteroidList.length; i++) 
          asteroidList[i].visible = false;
        for (i = 0; i < bulletList.length; i++) 
          bulletList[i].visible = false;
        shield.visible = false;
        ship.visible = false;
        damage.visible = false;
        life.visible = false;
        if (score > bestScore) bestScore = score; 
        txtScore.text = "BEST SCORE: " + int(bestScore);
      }
    }
  }
}
